# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio Global Search Record Resource Config."""
from typing import ClassVar

from flask_resources import (
    JSONDeserializer,
    JSONSerializer,
    RequestBodyParser,
    ResponseHandler,
)
from invenio_records_resources.resources import (
    RecordResourceConfig,
    SearchRequestArgsSchema,
)
from marshmallow import fields

from .serializers import GlobalSearchJSONSerializer


class GlobalSearchRecordResourceConfig(RecordResourceConfig):
    """GlobalSearchRecordResourceConfig."""

    blueprint_name = "global_search"

    url_prefix = "/global-search"

    default_accept_mimetype = "application/json"

    response_handlers: ClassVar = {
        "application/json": ResponseHandler(JSONSerializer()),
        "application/vnd.invenioglobalsearch.v1+json": ResponseHandler(
            GlobalSearchJSONSerializer(),
        ),
    }

    routes: ClassVar = {
        "search": "/search",
        "list": "",
    }

    # Request parsing
    request_args = SearchRequestArgsSchema
    request_view_args: ClassVar = {
        "pid_value": fields.Str(),
        "pid_type": fields.Str(),
    }
    request_headers: ClassVar = {
        "if_match": fields.Int(),
    }
    request_body_parsers: ClassVar = {
        "application/json": RequestBodyParser(JSONDeserializer()),
    }



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023-2024 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Global Search Services."""

from flask_principal import Identity
from invenio_pidstore.errors import PIDDoesNotExistError
from invenio_records_resources.services import RecordService
from invenio_records_resources.services.records.results import RecordItem
from invenio_records_resources.services.uow import UnitOfWork, unit_of_work


class GlobalSearchRecordService(RecordService):
    """Global search record service."""

    @unit_of_work()
    def create_or_update(
        self,
        identity: Identity,
        data: dict,
        uow: UnitOfWork | None = None,
        *,
        expand: bool = False,
    ) -> RecordItem:
        """Create a record.

        :param identity: Identity of user creating the record.
        :param data: Input data according to the data schema.
        """
        gs_pid = "gs-" + data["original"]["pid"]

        try:
            # only to check if a exists already
            self.record_cls.pid.resolve(gs_pid)

            return self.update(identity, id_=gs_pid, data=data)
        except PIDDoesNotExistError:
            self.record_cls.gs_pid = gs_pid
            return self._create(self.record_cls, identity, data, uow=uow, expand=expand)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Sphinx configuration."""

from datetime import datetime, timezone

from invenio_records_global_search import __version__

# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = "invenio-records-global-search"
project_copyright = f"{datetime.now(timezone.utc).year}, Graz University of Technology"
author = "Graz University of Technology"
release = __version__

language = "en"

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = []

templates_path = ["_templates"]
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]


# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = "alabaster"
# html_static_path = ['_static']



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Global Search Schemas."""

from invenio_records_resources.services.records.schema import BaseRecordSchema
from marshmallow.fields import Dict
from marshmallow_utils.fields import NestedAttribute

from .metadata import GlobalSearchMetadataSchema


class GlobalSearchRecordSchema(BaseRecordSchema):
    """GlobalSearchRecordSchema."""

    metadata = NestedAttribute(GlobalSearchMetadataSchema)

    original = Dict()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023-2024 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Global Search Metadata Schema."""

from marshmallow import Schema
from marshmallow.fields import List, String
from marshmallow_utils.fields import SanitizedUnicode


class GlobalSearchMetadataSchema(Schema):
    """GlobalSearchMetadataSchema."""

    contributors = List(String())
    titles = List(SanitizedUnicode(required=True))
    creators = List(String(required=True))
    identifiers = List(String(required=True))
    relations = List(String())
    rights = List(String())
    dates = List(String())
    subjects = List(String())
    descriptions = List(String())
    publishers = List(String())
    types = List(String())
    sources = List(String())
    languages = List(String())
    locations = List(String())
    formats = List(String())



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023-2024 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Original Component."""

from flask_principal import Identity
from invenio_records_resources.records.api import Record
from invenio_records_resources.services.records.components.base import ServiceComponent


class OriginalComponent(ServiceComponent):
    """Service component for metadata."""

    def create(
        self,
        _: Identity,
        data: dict | None = None,
        record: Record | None = None,
        errors: dict | None = None,  # noqa: ARG002
        **__: dict,
    ) -> None:
        """Inject parsed view to the record."""
        record.original = data.get("original", "")

    def update(
        self,
        _: Identity,
        data: dict | None = None,
        record: Record | None = None,
        **__: dict,
    ) -> None:
        """Inject parsed view to the record."""
        record.original = data.get("original", "")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Helper proxy to the state object."""

from flask import current_app
from werkzeug.local import LocalProxy

current_records_global_search = LocalProxy(
    lambda: current_app.extensions["invenio-records-global-search"],
)
"""Helper proxy to get the global search extension."""



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Configs."""

from invenio_i18n import gettext as _

from .services.facets import data_model, formats, publishers, rights, subjects, types

GLOBAL_SEARCH_FACETS = {
    "data_model": {
        "facet": data_model,
        "ui": {
            "field": "original.schema",
        },
    },
    "subjects": {
        "facet": subjects,
        "ui": {
            "field": "metadata.subjects",
        },
    },
    "publishers": {
        "facet": publishers,
        "ui": {
            "field": "metadata.publishers",
        },
    },
    "formats": {
        "facet": formats,
        "ui": {
            "field": "metadata.formats",
        },
    },
    "rights": {
        "facet": rights,
        "ui": {
            "field": "metadata.rights",
        },
    },
    "types": {
        "facet": types,
        "ui": {
            "field": "metadata.types",
        },
    },
}

GLOBAL_SEARCH_SORT_OPTIONS = {
    "bestmatch": {
        "title": _("Best match"),
        "fields": ["_score"],  # search defaults to desc on `_score` field
    },
    "newest": {
        "title": _("Newest"),
        "fields": ["-created"],
    },
}

GLOBAL_SEARCH_SEARCH = {
    "facets": ["data_model", "subjects", "publishers", "formats", "rights", "types"],
    "sort": [
        "bestmatch",
        "newest",
    ],
}

GLOBAL_SEARCH_BASE_TEMPLATE = "invenio_records_global_search/base.html"

GLOBAL_SEARCH_ORIGINAL_SCHEMAS = {}
"""Configure the original schema names."""



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023-2024 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Original system field."""

from invenio_records.systemfields.base import SystemField
from invenio_records_resources.records.api import Record


class OriginalField(SystemField):
    """OriginalField."""

    def __init__(
        self,
        key: str = "original",
        *,
        clear_none: bool = False,
        create_if_missing: bool = True,
    ) -> None:
        """Initialise the dict field.

        :param key: Key to set (dot notation supported).
        :param clear_none: Boolean to control if empty/None values should be
                           removed.
        :param create_if_missing: If a subkey is missing it will be created if
                                  this option is set to true.
        """
        self.clear_none = clear_none
        self.create_if_missing = create_if_missing
        super().__init__(key=key)

    def __set__(self, record: Record, value: dict | list | str) -> None:
        """Set."""
        self.set_dictkey(record, value, create_if_missing=self.create_if_missing)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Global Search Service Config."""

from typing import ClassVar

from invenio_records_resources.services import (
    RecordServiceConfig,
    SearchOptions,
    pagination_links,
)
from invenio_records_resources.services.base.config import (
    ConfiguratorMixin,
    FromConfig,
    FromConfigSearchOptions,
    SearchOptionsMixin,
)

from ..records import GlobalSearchRecord
from .components import DefaultRecordsComponents
from .facets import data_model, formats, publishers, rights, subjects, types
from .permissions import GlobalSearchRecordPermissionPolicy
from .schemas import GlobalSearchRecordSchema


class GlobalSearchSearchOptions(SearchOptions, SearchOptionsMixin):
    """Search options for record search."""

    facets: ClassVar = {
        "data_model": data_model,
        "subjects": subjects,
        "publishers": publishers,
        "formats": formats,
        "rights": rights,
        "types": types,
    }


class GlobalSearchRecordServiceConfig(RecordServiceConfig, ConfiguratorMixin):
    """Dublin Core record service config class."""

    record_cls = GlobalSearchRecord

    schema = FromConfig(
        "GLOBAL_SEARCH_SCHEMA",
        default=GlobalSearchRecordSchema,
    )

    permission_policy_cls = FromConfig(
        "GLOBAL_SEARCH_PERMISSION_POLICY",
        default=GlobalSearchRecordPermissionPolicy,
    )

    links_search = pagination_links("{+api}/global-search{?args*}")

    # Search
    search = FromConfigSearchOptions(
        "GLOBAL_SEARCH_SEARCH",
        "GLOBAL_SEARCH_SORT_OPTIONS",
        "GLOBAL_SEARCH_FACETS",
        search_option_cls=GlobalSearchSearchOptions,
    )

    components = FromConfig(
        "GLOBAL_SEARCH_RECORDS_SERVICE_COMPONENTS",
        default=DefaultRecordsComponents,
    )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Facets."""

from flask import current_app
from invenio_i18n import gettext as _
from invenio_records_resources.services.records.facets import TermsFacet


def data_model_labels(schemas: list) -> dict:
    """Create l10n names of the data model labels."""
    labels = current_app.config["GLOBAL_SEARCH_ORIGINAL_SCHEMAS"]
    out = {}
    for schema in schemas:
        out[schema] = labels.get(schema, {}).get("name_l10n", schema)
    return out


data_model = TermsFacet(
    field="original.schema",
    label=_("DataModel"),
    value_labels=data_model_labels,
)

subjects = TermsFacet(
    field="metadata.subjects",
    label=_("Subjects"),
)

publishers = TermsFacet(
    field="metadata.publishers",
    label=_("Publishers"),
)

formats = TermsFacet(
    field="metadata.formats",
    label=_("Formats"),
)

rights = TermsFacet(
    field="metadata.rights",
    label=_("Rights"),
    value_labels={
        "https://creativecommons.org/licenses/by/4.0": _("CC BY 4.0"),
        "https://creativecommons.org/licenses/by-sa/4.0": _("CC BY-SA 4.0"),
        "https://creativecommons.org/licenses/by-nd/4.0": _("CC BY-ND 4.0"),
        "https://creativecommons.org/licenses/by-nc/4.0": _("CC BY-NC 4.0"),
        "https://creativecommons.org/licenses/by-nc-sa/4.0": _("CC BY-NC-SA 4.0"),
        "https://creativecommons.org/licenses/by-nc-nd/4.0": _("CC BY-NC-ND 4.0"),
    },
)

types = TermsFacet(
    field="metadata.types",
    label=_("Types"),
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023-2024 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Records API."""

from invenio_records.systemfields import ConstantField, DictField
from invenio_records_resources.records.api import Record
from invenio_records_resources.records.systemfields import IndexField, PIDField
from typing_extensions import Self

from .models import GlobalSearchMetadata
from .systemfields import (
    GlobalSearchPIDFieldContext,
    GlobalSearchRecordProvider,
    OriginalField,
)


class GlobalSearchRecord(Record):
    """Dublin Core Record."""

    model_cls = GlobalSearchMetadata

    schema = ConstantField(
        "$schema",
        "local://global-search/records/global-search-record-v1.0.0.json",
    )

    index = IndexField(
        "global-search-records-record-v1.0.0",
        search_alias="global-search",
    )

    pids = DictField("pids")

    pid = PIDField(
        key="id",
        create=True,
        provider=GlobalSearchRecordProvider,
        context_cls=GlobalSearchPIDFieldContext,
    )

    original = OriginalField()

    gs_pid = None

    @classmethod
    def create(cls, data: dict, id_: str | None = None, **kwargs: dict) -> Self:
        """Create."""
        data["gs_pid"] = cls.gs_pid
        return super().create(data, id_, **kwargs)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""JS/CSS Webpack bundles for theme."""

from invenio_assets.webpack import WebpackThemeBundle

theme = WebpackThemeBundle(
    __name__,
    "assets",
    default="semantic-ui",
    themes={
        "semantic-ui": {
            "entry": {
                "invenio-records-global-search-theme": "./less/invenio_records_global_search/theme.less",
                "invenio-records-global-search-search": "./js/invenio_records_global_search/search/index.js",
            },
            "dependencies": {},
            "aliases": {
                "@js/invenio_records_global_search": "js/invenio_records_global_search",
                "@less/invenio_records_global_search": "less/invenio_records_global_search",
                "@translations/invenio_records_global_search": "translations/invenio_records_global_search",
            },
        },
    },
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio Global Search Record Resource."""

from flask_resources import route
from invenio_records_resources.resources import RecordResource


class GlobalSearchRecordResource(RecordResource):
    """GlobalSearchRecordResource."""

    def create_url_rules(self) -> list:
        """Create url rules for record resource."""
        routes = self.config.routes

        return [
            route("GET", routes["list"], self.search),
        ]



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""System Fields for Global Search records."""


from .context import GlobalSearchPIDFieldContext
from .original import OriginalField
from .providers import GlobalSearchRecordProvider

__all__ = ("GlobalSearchRecordProvider", "GlobalSearchPIDFieldContext", "OriginalField")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""CLI."""

import click
from flask import current_app
from flask.cli import with_appcontext
from invenio_access.permissions import system_identity

from .proxies import current_records_global_search


@click.group()
def global_search() -> None:
    """CLI-group for "invenio record global search" commands."""


@global_search.command()
@with_appcontext
def rebuild_index() -> None:
    """Reindex all records."""
    click.secho("Reindexing records of global search...", fg="green")

    service = current_records_global_search.records_service
    service.rebuild_index(identity=system_identity)

    click.secho("Reindexed records!", fg="green")


@global_search.command()
@with_appcontext
def rebuild_database() -> None:
    """Rebuild database."""
    for func in current_app.config.get("GLOBAL_SEARCH_REBUILD_DATABASE", []):
        func()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Dublin Core data model for InvenioRDM."""

from .ext import InvenioRecordsGlobalSearch
from .proxies import current_records_global_search
from .records import GlobalSearchRecord

__version__ = "0.3.0"

__all__ = (
    "__version__",
    "InvenioRecordsGlobalSearch",
    "GlobalSearchRecord",
    "current_records_global_search",
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023-2024 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Global search data model for InvenioRDM."""

from flask import Flask

from . import config
from .resources import GlobalSearchRecordResource, GlobalSearchRecordResourceConfig
from .services import GlobalSearchRecordService, GlobalSearchRecordServiceConfig


class InvenioRecordsGlobalSearch:
    """Invenio-Records-Global-Search extension."""

    def __init__(self, app: Flask = None) -> None:
        """Construct InvenioRecordsGlobalSearch."""
        if app:
            self.init_app(app)

    def init_app(self, app: Flask) -> None:
        """Flask application initialization."""
        self.init_config(app)
        self.init_services(app)
        self.init_resources()
        app.extensions["invenio-records-global-search"] = self

    def init_config(self, app: Flask) -> None:
        """Initialize configuration."""
        for k in dir(config):
            if k.startswith("GLOBAL_SEARCH_"):
                app.config.setdefault(k, getattr(config, k))

    def init_services(self, app: Flask) -> None:
        """Initialize services."""
        service_config = GlobalSearchRecordServiceConfig.build(app)
        self.records_service = GlobalSearchRecordService(config=service_config)

    def init_resources(self) -> None:
        """Initialize resources."""
        self.records_resource = GlobalSearchRecordResource(
            service=self.records_service,
            config=GlobalSearchRecordResourceConfig,
        )


def finalize_app(app: Flask) -> None:
    """Finalize app."""
    init(app)


def init(app: Flask) -> None:
    """Init app."""
    # Register services - cannot be done in extension because
    # Invenio-Records-Resources might not have been initialized.
    ext = app.extensions["invenio-records-global-search"]
    sregistry = app.extensions["invenio-records-resources"].registry
    sregistry.register(ext.records_service, service_id="global-search-record")

    iregistry = app.extensions["invenio-indexer"].registry
    iregistry.register(ext.records_service.indexer, indexer_id="global-search-record")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""High-level API for working with Dublin Core records, pids, search."""

from .config import GlobalSearchRecordServiceConfig
from .services import GlobalSearchRecordService

__all__ = ("GlobalSearchRecordService", "GlobalSearchRecordServiceConfig")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Components."""


from invenio_records_resources.services.records.components import MetadataComponent

from .original import OriginalComponent

DefaultRecordsComponents = [
    MetadataComponent,
    OriginalComponent,
]


__all__ = ("DefaultRecordsComponents",)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023-2024 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio Global Search Record Resource schema."""

from flask import current_app
from marshmallow import Schema, fields
from marshmallow_utils.fields import SanitizedUnicode


def access_status(_: dict) -> dict:
    """Access status."""
    return {
        "id": "open",
        "icon": "unlock",
        "title_l10n": "Open",
        "description_l10n": "Open",
    }


def created_date_l10n_long(obj: dict) -> str:
    """Create date l10n long."""
    if "dates" in obj["metadata"] and len(obj["metadata"]["dates"]) > 0:
        return obj["metadata"]["dates"][0]
    return "N/A"


class OriginalSchema(Schema):
    """Original Schema."""

    view = fields.String(attribute="view")
    schema_l10n = fields.Method("get_schema_l10n")

    def get_schema_l10n(self, obj: dict) -> str:
        """Get schema l10n."""
        schemas = current_app.config.get("GLOBAL_SEARCH_ORIGINAL_SCHEMAS", {})
        for schema_name, schema in schemas.items():
            if schema_name == obj["schema"]:
                return schema["name_l10n"]
        return ""


class GlobalSearchSchema(Schema):
    """Schema for dumping extra information for the global search record."""

    id = SanitizedUnicode(data_key="id", attribute="id")

    access_status = fields.Function(access_status)

    original = fields.Nested(OriginalSchema, attribute="original")

    created_date_l10n_long = fields.Function(created_date_l10n_long)

    class Meta:
        """Meta class to accept unknwon fields."""

        additional = ("metadata",)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023-2024 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Global Search PID providers."""

from typing import Self

from invenio_pidstore.providers.recordid_v2 import RecordIdProviderV2


class GlobalSearchRecordProvider(RecordIdProviderV2):
    """GlobalSearchRecordProvider."""

    pid_type = "gs"

    @classmethod
    def generate_id(cls: Self, _: dict | None = None) -> str:
        """Generate id."""
        return cls.gs_pid

    @classmethod
    def create(
        cls: Self,
        object_type: str | None = None,
        object_uuid: str | None = None,
        options: dict | None = None,
        **kwargs: dict,
    ) -> Self:
        """Create."""
        cls.gs_pid = kwargs["record"]["gs_pid"]
        return super().create(object_type, object_uuid, options, **kwargs)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio Dublin Core Records module to create REST APIs."""

from .config import GlobalSearchRecordResourceConfig
from .resources import GlobalSearchRecordResource

__all__ = ("GlobalSearchRecordResource", "GlobalSearchRecordResourceConfig")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023-2024 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Global Search permission policy."""
from typing import ClassVar

from invenio_records_permissions.generators import AnyUser, RecordOwners, SystemProcess
from invenio_records_permissions.policies.records import RecordPermissionPolicy


class GlobalSearchRecordPermissionPolicy(RecordPermissionPolicy):
    """GlobalSearchRecordPermissionPolicy."""

    can_create: ClassVar = [AnyUser(), SystemProcess()]

    can_search: ClassVar = [AnyUser(), SystemProcess()]
    can_read: ClassVar = [AnyUser(), RecordOwners()]

    can_update: ClassVar = [AnyUser(), SystemProcess()]



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio Dublin Core Record Resource serializers."""


from flask_resources import BaseListSchema, JSONSerializer
from flask_resources.serializers import MarshmallowSerializer

from .schema import GlobalSearchSchema


class GlobalSearchJSONSerializer(MarshmallowSerializer):
    """Dublin Core base serializer implementation."""

    def __init__(self) -> None:
        """Dublin Core Base Serializer Constructor.

        :param schema_cls: Default GlobalSearchSchema
        :param options: Json encoding options.
        """
        super().__init__(
            format_serializer_cls=JSONSerializer,
            object_schema_cls=GlobalSearchSchema,
            list_schema_cls=BaseListSchema,
        )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023-2024 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Views."""

from functools import partial

from flask import Blueprint, Flask, current_app, render_template
from invenio_search_ui.searchconfig import search_app_config


def create_blueprint(_: Flask) -> Blueprint:
    """Create Blueprint."""
    routes = {
        "record-search": "/search",
    }

    blueprint = Blueprint(
        "invenio_records_global_search",
        __name__,
        template_folder="templates",
        url_prefix="/global-search",
    )
    blueprint.add_url_rule(routes["record-search"], view_func=search)
    blueprint.app_context_processor(search_app_context)

    return blueprint


def search_app_context() -> dict:
    """Global search search app context."""
    return {
        "search_app_global_search_config": partial(
            search_app_config,
            "GLOBAL_SEARCH_SEARCH",
            current_app.config["GLOBAL_SEARCH_FACETS"],
            current_app.config["GLOBAL_SEARCH_SORT_OPTIONS"],
            "/api/global-search",
            {"Accept": "application/vnd.invenioglobalsearch.v1+json"},
            app_id="GlobalSearchRecords.Search",
        ),
    }


def search() -> str:
    """Search help guide."""
    return render_template("invenio_records_global_search/search/search.html")


def create_record_bp(app: Flask) -> Blueprint:
    """Create records blueprint."""
    return app.extensions[
        "invenio-records-global-search"
    ].records_resource.as_blueprint()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Graz University of Technology.
#
# invenio-records-global-search is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Dublin core schemas."""

from .record import GlobalSearchRecordSchema

__all__ = ("GlobalSearchRecordSchema",)
